# -*- coding: utf-8 -*-
"""
Created on Wed Jan 22 10:07:12 2025

@author: mstep
"""

#%%

import tkinter as tk
from PIL import Image, ImageDraw, ImageTk

class SierpinskiCarpetApp:
    def __init__(self):
        self.window = tk.Tk()

        self.window.title('sierpinski_carpet')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('300x400')

        # Drawing parameters.
        wid = 300
        hgt = 300
        margin = 10

        # Parameters.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, padx=margin, pady=(margin,0),
            expand=False, fill=tk.X)
        label = tk.Label(frame, text='Depth:')
        label.pack(side=tk.LEFT)
        self.depth_var = tk.StringVar(value='3')
        self.depth_spinbox = tk.Spinbox(frame, from_=0, to_=6, increment=1,
            justify=tk.RIGHT, width=3, textvariable=self.depth_var,
            state='readonly')
        self.depth_spinbox.pack(side=tk.LEFT)
        button = tk.Button(frame, text='Draw', width=6,
            command=self.draw_carpet)
        button.pack(side=tk.LEFT, padx=20)

        # Canvas for drawing the carpet.
        self.canvas = tk.Canvas(self.window, bg='white',
            width=wid, height=hgt, borderwidth=0, highlightthickness=0)
        self.canvas.pack(side=tk.TOP, anchor=tk.NW,
            padx=margin, pady=(5,margin),
            expand=True, fill=tk.BOTH)

        self.window.bind('<Return>', lambda _: self.draw_carpet())

        # Size the window to fit.
        self.window.geometry(f'{wid + 2 * margin}x{hgt + 2 * margin}')

        # Display the window.
        self.depth_spinbox.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def draw_carpet(self):
        '''Draw the carpet.'''
        # Get parameters.
        margin = 10
        depth = int(self.depth_var.get())
        make_square = True
        if make_square:
            wid = hgt = min(self.canvas.winfo_width(), self.canvas.winfo_height())
        else:
            wid = self.canvas.winfo_width()
            hgt = self.canvas.winfo_height()
        wid -= 2 * margin
        hgt -= 2 * margin

        # Generate the carpet.
        carpet = self.make_carpet(depth, wid, hgt, 'blue', 'yellow')

        # Display the carpet.
        self.canvas.delete(tk.ALL)
        self.carpet_photo_image = ImageTk.PhotoImage(carpet)
        cx = self.canvas.winfo_width() / 2
        cy = self.canvas.winfo_height() / 2
        self.canvas.create_image(cx, cy, image=self.carpet_photo_image)

        # Save the carpet.
        carpet.save('carpet.png')

    def make_carpet(self, depth, wid, hgt, fg, bg):
        '''Make the carpet image.'''
        # Make the image, initially filled with the fg color.
        image = Image.new('RGB', (wid, hgt), fg)
        dr = ImageDraw.Draw(image)

        # Recursively draw the carpet.
        self.remove_center(depth, dr, bg, 0, 0, wid, wid)
        return image

    def remove_center(self, depth, dr, fill, xmin, ymin, xmax, ymax):
        '''Clear the center of this area.'''
        # Divide the area into 9 pieces.
        x3 = (xmax - xmin) / 3    # 1/3 of the width and height.
        y3 = (ymax - ymin) / 3

        # Clear the center.
        dr.rectangle(xy=(xmin + x3, ymin + y3, xmin + 2 * x3, ymin + 2 * y3),
            fill=fill)

        # See if should recurse.
        depth -= 1
        if depth >= 0:
            for r in range(3):
                for c in range(3):
                    if r != 1 or c != 1:
                        self.remove_center(depth, dr, fill,
                            xmin + x3 * c,       ymin + y3 * r,
                            xmin + x3 * (c + 1), ymin + y3 * (r + 1))

#%%
SierpinskiCarpetApp()
print('Done')
